@@ -4,26 +4,21 @@
  * The context structure is the internal state maintained by the pam-krb5
  * module between calls to the various public interfaces.
  *
- * Copyright 2005, 2006, 2007, 2008 Russ Allbery <rra@debian.org>
+ * Copyright 2005, 2006, 2007, 2008, 2009 Russ Allbery <rra@debian.org>
  * Copyright 2005 Andres Salomon <dilinger@debian.org>
  * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ *
  * See LICENSE for licensing terms.
  */
 
-#include "config.h"
+#include <config.h>
+#include <portable/pam.h>
 
 #include <errno.h>
-#ifdef HAVE_SECURITY_PAM_APPL_H
-# include <security/pam_appl.h>
-# include <security/pam_modules.h>
-#elif HAVE_PAM_PAM_APPL_H
-# include <pam/pam_appl.h>
-# include <pam/pam_modules.h>
-#endif
 #include <stdlib.h>
 #include <string.h>
 
-#include "internal.h"
+#include <internal.h>
 
 /* Solaris doesn't have these. */
 #ifndef PAM_CONV_AGAIN
@@ -65,9 +60,12 @@ pamk5_context_new(struct pam_args *args)
         goto done;
     }
     ctx->name = strdup(name);
-    retval = krb5_init_context(&ctx->context);
+    if (pamk5_compat_issetugid())
+        retval = pamk5_compat_secure_context(&ctx->context);
+    else
+        retval = krb5_init_context(&ctx->context);
     if (retval != 0) {
-        pamk5_error_krb5(args, "krb5_init_context", retval);
+        pamk5_err_krb5(args, retval, "krb5_init_context failed");
         retval = PAM_SERVICE_ERR;
         goto done;
     }
@@ -76,7 +74,7 @@ pamk5_context_new(struct pam_args *args)
     if (args->realm != NULL) {
         retval = krb5_set_default_realm(ctx->context, args->realm);
         if (retval != 0) {
-            pamk5_error_krb5(args, "cannot set default realm", retval);
+            pamk5_err_krb5(args, retval, "cannot set default realm");
             retval = PAM_SERVICE_ERR;
             goto done;
         }
