@@ -5,19 +5,20 @@
  * internal functions.  Retrieves configuration information from krb5.conf and
  * parses the PAM configuration.
  *
- * Copyright 2005, 2006, 2007, 2008 Russ Allbery <rra@debian.org>
+ * Copyright 2005, 2006, 2007, 2008, 2009 Russ Allbery <rra@debian.org>
  * Copyright 2005 Andres Salomon <dilinger@debian.org>
  * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ *
  * See LICENSE for licensing terms.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <krb5.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "internal.h"
+#include <internal.h>
 
 /*
  * Not all platforms have this, so just implement it ourselves.  Copy a
@@ -162,7 +163,7 @@ default_time(struct pam_args *args, krb5_context c, const char *opt,
         ret = krb5_string_to_deltat(tmp, result);
         if (ret != 0) {
             message = pamk5_compat_get_error(c, ret);
-            pamk5_error(NULL, "bad time value for %s: %s", opt, message);
+            pamk5_err(args, "bad time value for %s: %s", opt, message);
             pamk5_compat_free_error(c, message);
             *result = defval;
         }
@@ -275,7 +276,10 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
      * proceed; we'll die soon enough later and this way we'll die after we
      * know whether to debug things.
      */
-    retval = krb5_init_context(&c);
+    if (pamk5_compat_issetugid())
+        retval = pamk5_compat_secure_context(&c);
+    else
+        retval = krb5_init_context(&c);
     if (retval != 0)
         c = NULL;
     if (c != NULL) {
@@ -283,6 +287,7 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
             krb5_get_default_realm(c, &args->realm);
         if (args->realm != NULL)
             pamk5_compat_set_realm(args, args->realm);
+        default_string(args, c, "alt_auth_map", NULL, &args->alt_auth_map);
         default_string(args, c, "banner", "Kerberos", &args->banner);
         default_string(args, c, "ccache", NULL, &args->ccache);
         default_string(args, c, "ccache_dir", "FILE:/tmp", &args->ccache_dir);
@@ -290,12 +295,15 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
         default_boolean(args, c, "debug", 0, &args->debug);
         default_boolean(args, c, "defer_pwchange", 0, &args->defer_pwchange);
         default_boolean(args, c, "expose_account", 0, &args->expose_account);
+        default_boolean(args, c, "fail_pwchange", 0, &args->fail_pwchange);
+        default_boolean(args, c, "force_alt_auth", 0, &args->force_alt_auth);
         default_boolean(args, c, "force_pwchange", 0, &args->force_pwchange);
         default_boolean(args, c, "forwardable", 0, &args->forwardable);
         default_boolean(args, c, "ignore_k5login", 0, &args->ignore_k5login);
         default_boolean(args, c, "ignore_root", 0, &args->ignore_root);
         default_string(args, c, "keytab", NULL, &args->keytab);
         default_number(args, c, "minimum_uid", 0, &args->minimum_uid);
+        default_boolean(args, c, "only_alt_auth", 0, &args->only_alt_auth);
         default_string(args, c, "pkinit_anchors", NULL, &args->pkinit_anchors);
         default_boolean(args, c, "pkinit_prompt", 0, &args->pkinit_prompt);
         default_string(args, c, "pkinit_user", NULL, &args->pkinit_user);
@@ -323,7 +331,12 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
      * sense in krb5.conf.
      */
     for (i = 0; i < argc; i++) {
-        if (strncmp(argv[i], "banner=", 7) == 0) {
+        if (strncmp(argv[i], "alt_auth_map=", 12) == 0) {
+            if (args->banner != NULL)
+                free(args->alt_auth_map);
+            args->alt_auth_map = strdup(&argv[i][strlen("alt_auth_map=")]);
+        }
+        else if(strncmp(argv[i], "banner=", 7) == 0) {
             if (args->banner != NULL)
                 free(args->banner);
             args->banner = strdup(&argv[i][strlen("banner=")]);
@@ -346,8 +359,14 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
             args->defer_pwchange = 1;
         else if (strcmp(argv[i], "expose_account") == 0)
             args->expose_account = 1;
+        else if (strcmp(argv[i], "fail_pwchange") == 0)
+            args->fail_pwchange = 1;
+        else if (strcmp(argv[i], "force_first_pass") == 0)
+            args->force_first_pass = 1;
         else if (strcmp(argv[i], "force_pwchange") == 0)
             args->force_pwchange = 1;
+        else if (strcmp(argv[i], "force_alt_auth") == 0)
+            args->force_alt_auth = 1;
         else if (strcmp(argv[i], "forwardable") == 0)
             args->forwardable = 1;
         else if (strcmp(argv[i], "ignore_k5login") == 0)
@@ -363,6 +382,8 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
             args->minimum_uid = atoi(&argv[i][strlen("minimum_uid=")]);
         else if (strcmp(argv[i], "no_ccache") == 0)
             args->no_ccache = 1;
+        else if (strcmp(argv[i], "only_alt_auth") == 0)
+            args->only_alt_auth = 1;
         else if (strncmp(argv[i], "pkinit_anchors=", 15) == 0) {
             if (args->pkinit_anchors != NULL)
                 free(args->pkinit_anchors);
@@ -416,7 +437,7 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
         else if (strcmp(argv[i], "use_pkinit") == 0)
             args->use_pkinit = 1;
         else
-            pamk5_error(NULL, "unknown option %s", argv[i]);
+            pamk5_err(NULL, "unknown option %s", argv[i]);
     }
     if (flags & PAM_SILENT)
         args->silent = 1;
@@ -427,18 +448,18 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
         args->banner = NULL;
     }
 
-    /* Sanity-check try_first_pass, use_first_pass, and use_authtok. */
-    if (args->use_authtok && args->try_first_pass) {
-        pamk5_error(NULL, "use_authtok set, ignoring try_first_pass");
+    /* Sanity-check try_first_pass, use_first_pass, and force_first_pass. */
+    if (args->force_first_pass && args->try_first_pass) {
+        pamk5_err(NULL, "force_first_pass set, ignoring try_first_pass");
         args->try_first_pass = 0;
         args->use_first_pass = 0;
     }
-    if (args->use_authtok && args->use_first_pass) {
-        pamk5_error(NULL, "use_authtok set, ignoring use_first_pass");
+    if (args->force_first_pass && args->use_first_pass) {
+        pamk5_err(NULL, "force_first_pass set, ignoring use_first_pass");
         args->use_first_pass = 0;
     }
     if (args->use_first_pass && args->try_first_pass) {
-        pamk5_error(NULL, "use_first_pass set, ignoring try_first_pass");
+        pamk5_err(NULL, "use_first_pass set, ignoring try_first_pass");
         args->try_first_pass = 0;
     }
 
@@ -462,11 +483,11 @@ pamk5_args_parse(pam_handle_t *pamh, int flags, int argc, const char **argv)
 #ifndef HAVE_KRB5_GET_INIT_CREDS_OPT_SET_PKINIT
 # ifndef HAVE_KRB5_GET_INIT_CREDS_OPT_SET_PA
     if (args->try_pkinit)
-	pamk5_error(NULL, "try_pkinit requested but PKINIT not available");
+	pamk5_err(NULL, "try_pkinit requested but PKINIT not available");
 # endif
     if (args->use_pkinit)
-	pamk5_error(NULL, "use_pkinit requested but PKINIT not available"
-                    " or cannot be enforced");
+	pamk5_err(NULL, "use_pkinit requested but PKINIT not available or"
+                  " cannot be enforced");
 #endif
 
     return args;
